package se.chalmers.tdv.datalog.testing;

import se.chalmers.tdv.datalog.Argument;
import se.chalmers.tdv.datalog.Atom;
import se.chalmers.tdv.datalog.Fact;
import se.chalmers.tdv.datalog.Predicate;
import se.chalmers.tdv.datalog.Program;
import se.chalmers.tdv.datalog.Rule;
import se.chalmers.tdv.datalog.Value;
import se.chalmers.tdv.datalog.Variable;

public class Bug {

	public static void main(String[] args) {
		Predicate parent = new Predicate("parent");
		Predicate ancestor = new Predicate("ancestor");
		Argument varX = Argument.variable(new Variable("X"));
		Argument varY = Argument.variable(new Variable("Y"));
		Argument varZ = Argument.variable(new Variable("Z"));
		Value alice = new Value("alice");
		Value bob = new Value("bob");
		Value charlie = new Value("charlie");
		Value chuck = new Value("chuck");
		Value dave = new Value("dave");

		/* Rule: ancestor(X,Y) :- parent(X,Y). */
		Rule r1 = new Rule(new Atom(ancestor, varX, varY), new Atom(parent,
				varX, varY));
		/* Rule: ancestor(X,Y) :- ancestor(X,Z), ancestor(Z,Y). */
		Rule r2 = new Rule(new Atom(ancestor, varX, varY), new Atom(ancestor,
				varX, varZ), new Atom(ancestor, varZ, varY));

		/*
		 * Database: parent(alice, bob), parent(bob, charlie), parent(bob,
		 * chuck), parent(chuck, dave)
		 */
		Fact[] database = new Fact[] { new Fact(parent, alice, bob),
				new Fact(parent, bob, charlie), new Fact(parent, bob, chuck),
				new Fact(parent, chuck, dave) };

		Program p = new Program(r1, r2);

		// Query: ancestor(alice, ?)
		Fact[] allFacts = p.query(new Atom(ancestor, Argument.value(alice),
				varX), database);

		System.out.println("Derived " + allFacts.length + " facts");
		for (Fact f : allFacts) {
			System.out.println(f.toString());
		}
	}

}
